@@ -2,7 +2,7 @@ import os
 import logging
 from enum import Enum
 from datetime import datetime
-from time import sleep
+from time import sleep, time
 
 from .errors import tb_str, SimulationError
 from .system import NumerousSystem
@@ -47,20 +47,29 @@ class NumerousBaseJob:
         self.last_print = {}
         self._t0 = None
         self._exit_code = None
+        # Wrapper to allow specialized instanciation of components
+        self.wrapper = None
 
     def _post_init(self):  # This is called before _run_job from the app.loop method
         timeout = 120
         t, states = self._load_states()
         dt = self.app.nc.params.get('dt_simulation', 60)
+
+        #Allow dt parameter to be a time value from front-end
+        if isinstance(dt, dict):
+            dt = dt['value']
+
         t_stop = self.app.end_time if not self.app.subscribe else 0
         self._t0 = t
 
+
         self.input = self.app.nc.get_inputs(self.app.scenario, t0=t, te=t_stop, dt=dt,
-                                            tag_prefix='', tag_separator='.',
+                                            tag_prefix='',
                                             timeout=timeout)
 
+
         self.system = NumerousSystem(self.app.nc, self.app.scenario, self.app.files, self.app.start_time, t_stop,
-                                     self.app.model_folder, self, states, t, dt, self.input)
+                                     self.app.model_folder, self, states, t, dt, self.input, wrapper=self.wrapper)
 
     def serialize_states(self, t: float = None):
         """
@@ -118,9 +127,52 @@ class NumerousBaseJob:
             exit_code = ExitCode.COMPLETED
         self._exit_code = exit_code
 
-    def run_job(self):
+    def run(self):
+        #Overwrite this method to create the simplest job
         raise NotImplementedError
 
+    def run_job(self):
+        # This is the classic pipelines approach - well suited for digital twins
+        # Run simulation in a loop
+        self.logger.debug(f'entering loop')
+
+        cause = "no cause"
+        details = ""
+        i = 0
+        last_data = None
+        data = None
+        t = None
+        try:
+            self.app.nc.set_scenario_progress('bootup', ScenarioStatus.INITIALZING, 0.0, force=True)
+            self.app.terminate.wait(timeout=5)
+            self.app.nc.set_scenario_progress('waiting for initial data', ScenarioStatus.INITIALZING, 0.0, force=True)
+
+            self.run()
+        except Exception as e:
+            from .errors import tb_str
+            self.logger.error(f'numerous_app crashed: {tb_str(e)}')
+            self.logger.debug(f'previous data: {last_data}')
+            self.logger.debug(f'data at crash: {data}')
+            cause = 'exception'
+            details = {"error message": tb_str(e), "previous_data": last_data, "data at crash": data}
+            self.app.status = -1
+            self.app.message = "app error - see logs"
+        finally:
+            if self.app.status == 0:
+                self.app.terminate.set()
+                cause = "forcefully terminated"
+            elif self.app.status == 1:
+                self.app.terminate.set()
+                cause = "completed"
+            elif self.app.status == 2:
+                self.app.terminate.set()
+                cause = 'hibernating'
+            if hasattr(self, 'output'):
+                self.output.close()
+            self._save_states(t, cause, details)
+            self.logger.warning('job terminated')
+
+
 
 class NumerousReportJob(NumerousBaseJob):
     def __init__(self):
@@ -152,6 +204,7 @@ class NumerousSimulationJob(NumerousBaseJob):
     def __init__(self):
         super(NumerousSimulationJob, self).__init__()
         self.system = None
+        #Specifies whether or not a job will save its results
         self.align_outputs_to_next_timestep = True
         self.logger = logging.getLogger('numerous-simulation-job')
         self.logger.setLevel(level=LOG_LEVEL)
@@ -192,6 +245,11 @@ class NumerousSimulationJob(NumerousBaseJob):
             t_stop = self.system.end_time
             t = self._t0
 
+            #Increasingly longer spacing between log updates to 60 s
+            last_logged = -1000
+            dt_log_max = 60
+            dt_log = 1
+
             while True:
                 if self.app.nc.terminate_event.is_set():
                     exit_code = ExitCode.TERMINATED
@@ -264,7 +322,8 @@ class NumerousSimulationJob(NumerousBaseJob):
                     if initial_output:
                         if "_index" not in initial_output:
                             initial_output.update({'_index': 0})
-                        self.output.write_row(initial_output)
+
+                            self.output.write_row(initial_output)
 
                 try:
                     tnew, outputs = self.step(t, self.system.dt)
@@ -274,6 +333,7 @@ class NumerousSimulationJob(NumerousBaseJob):
                         self.app.nc.set_timeseries_meta_data([{"name": tag} for tag in outputs.keys()],
                                                          offset=self.app.start_time)
                         self.init = False
+
                     if not self.align_outputs_to_next_timestep:
                         outputs.update({'_index': t - self.app.start_time})
                     else:
@@ -288,7 +348,12 @@ class NumerousSimulationJob(NumerousBaseJob):
                 # save output
                 self.output.write_row(outputs)
 
-                self.logger.info(f'Calculation step. Time is now {t}. completed: {completed}')
+                if time() - last_logged > dt_log:
+                    last_logged = time()
+                    dt_log = min(dt_log_max, dt_log*2)
+
+                    self.logger.info(f'Calculation step. Time is now {t}. completed: {completed}')
+
                 last_data = data
 
                 if (t >= t_stop) and (t_stop > 0):
